@@ -23,7 +23,7 @@ from pkg_resources import iter_entry_points, EntryPoint
 from pluginlib.exceptions import PluginImportError, EntryPointWarning
 from pluginlib._objects import BlacklistEntry
 from pluginlib._parent import get_plugins
-from pluginlib._util import BASESTRING, LOGGER, NoneType, raise_with_traceback
+from pluginlib._util import BASESTRING, LOGGER, NoneType, PY34, raise_with_traceback
 
 try:
     from collections.abc import Iterable
@@ -123,10 +123,7 @@ def _import_module(name, path=None):
 
     LOGGER.debug('Attempting to load module %s from %s', name, path)
     try:
-        if epoint:
-            mod = epoint.load()
-        else:
-            mod = importlib.import_module(name)
+        mod = epoint.load() if epoint else importlib.import_module(name)
 
     except Exception as e:  # pylint: disable=broad-except
         _raise_friendly_exception(e, name, path)
@@ -174,7 +171,8 @@ def _recursive_path_import(path, prefix_package):
     # Walk path
     for root, dirs, files in os.walk(path):
         # If root is a Python module, we won't walk any farther down it
-        if '__init__.py' in files and sys.version_info[0] > 2:
+        # find_module() seems to be recursive in Python 3
+        if '__init__.py' in files and (3, 0) <= sys.version_info[:2] < (3, 5):  # pragma: no cover
             dirs.clear()
             if root != path:
                 continue
@@ -191,7 +189,14 @@ def _recursive_path_import(path, prefix_package):
         for finder, name, is_pkg in pkgutil.walk_packages([root], prefix=prefix):
             LOGGER.debug('Attempting to load module %s from %s', name, finder.path)
             try:
-                finder.find_module(name).load_module(name)
+                # find_module() was deprecated in 3.4, but module_from_spec wasn't available
+                if PY34:  # pragma: no cover
+                    finder.find_module(name).load_module(name)
+                else:
+                    spec = finder.find_spec(name)
+                    module = importlib.util.module_from_spec(spec)
+                    spec.loader.exec_module(module)
+
             except Exception as e:  # pylint: disable=broad-except
                 _raise_friendly_exception(e, name, root)
 
@@ -280,6 +285,7 @@ class PluginLoader(object):
                     try:
                         entry = BlacklistEntry(*entry)
                     except (AttributeError, TypeError) as e:
+                        # pylint: disable=raise-missing-from
                         raise AttributeError("Invalid blacklist entry '%s': %s " % (entry, e))
                 else:
                     raise AttributeError("Invalid blacklist entry '%s': Not an iterable" % entry)
